#########读取并查看数据结构
data <- read.csv("dat_alternation.csv", header = TRUE)
  # 可以在读取数据的同时把所有分类变量批量转化为因子:
  # stringsAsFactors=TRUE

# 查看读取的数据集
head(data)
str(data)

# 由于这批数据标注时对二分类变量采取的是0-1编码方式，
# 若读取数据时进行批量因子化，会把这些分类变量当作数值处理，
# 因此需要手动分别对每个变量进行因子化
data$Dative <- as.factor(data$Dative)
data$ReciAnim <- as.factor(data$ReciAnim)
data$ThemeAnim <- as.factor(data$ThemeAnim)
data$ReciAcc <- as.factor(data$ReciAcc)
data$ThemeAcc <- as.factor(data$ThemeAcc)
data$ReciPron <- as.factor(data$ReciPron)
data$ThemePron <- as.factor(data$ThemePron)
data$LengthDiff <- as.factor(data$LengthDiff)
data$ReciDef <- as.factor(data$ReciDef)
data$ThemeDef <- as.factor(data$ThemeDef)
data$ThemeCon <- as.factor(data$ThemeCon)
data$ReciCom <- as.factor(data$ReciCom)
data$ThemeCom <- as.factor(data$ThemeCom)

# 也可以用lapply函数批量操作
# data[ , 1:13] <- lapply(data[ , 1:13], as.factor)

# 查看并确认数据集已完成因子化
str(data)


#########二分类任务：英语中的与格交替
# 固定效应模型

## 划分训练集和测试集
set.seed(1234)

# 若没有安装过`dplyr`包，需要先安装后调用
# install.packages("dplyr")
library(dplyr)

train.data <- sample_frac(data, 0.8)
sample.id <- as.numeric(rownames(train.data))
test.data <- data[-sample.id,]

## 拟合模型
dat.model1<-glm(Dative~ReciAnim+ThemeAnim
                +ReciAcc+ThemeAcc
                +ReciPron+ThemePron
                +LengthDiff
                +ReciDef+ThemeDef
                +ThemeCon
                +ReciCom+ThemeCom, 
                family=binomial(), data=train.data)
summary(dat.model1)

exp(-4.2542)
exp(-4.2542)/(1+ exp(-4.2542))

#### 手动筛选自变量：删除不显著变量后再次拟合模型、并比较两次模型是否具有显著差异
dat.model0<-glm(Dative~ReciAnim+ThemeAnim
                +ReciAcc+ThemeAcc
                +ReciPron+ThemePron
                +LengthDiff
                +ThemeCon
                +ThemeCom, 
                family=binomial(), data=train.data)
anova(dat.model0,dat.model1,test = "Chisq")

#### 自动筛选变量：逐步回归
dat.model2<-step(dat.model1, direction="backward") # 向后逐步回归
summary(dat.model2)

dat.model3<-step(dat.model1, direction="forward") # 向前逐步回归
summary(dat.model3)

dat.model4<-step(dat.model1, direction="both") # 双向逐步回归
summary(dat.model4)

#### 计算回归系数的95%置信区间
confint(dat.model2)

#### 绘制效应图
install.packages("effects")
library(effects)

plot(predictorEffect("ReciAnim",dat.model2),
     axes=list(grid=TRUE,
               y=list(lab="Probability for double-object construction")),
     type="response")

## 模型优度评估
#### 计算混淆矩阵
install.packages("caret")
library(caret)

predictions <- predict(dat.model2, newdata=test.data, type="response")
predictions.output <- ifelse(predictions>0.5, 1, 0)
predictions.factor <- as.factor(predictions.output)

confusion_matrix<-confusionMatrix(data=predictions.factor, 
                                  reference=test.data$Dative,
                                  positive = "1")
summary(predictions.factor)
summary(test.data$Dative)

#### 绘制ROC曲线
install.packages("pROC")
library(pROC)
roc_curve<-roc(test.data$Dative,predictions)
plot(roc_curve, print.auc=TRUE,auc.polygon=TRUE,
     grid=TRUE,
     auc.polygon.col="lightgrey",
     identity.lwd=2,identity.lty=2,
     print.thres = "best", print.thres.best.method = "closest.topleft")

#### 计算模型的决定系数
install.packages("rsq")
library(rsq)
McFadden_r_squared<-rsq.lr(dat.model2) #McFadden R2
Nagelkerke_r_squared <- rsq.n(dat.model2) #Nagelkerke R2

## 模型诊断
#### 计算方差膨胀因子
install.packages("rms")
library(rms)
vif(dat.model2)

#### 查看样本量
summary(train.data)

#### 查看样本数据是否符合要求
install.packages("car")
library(car)
influencePlot(dat.model2)
train.data[c(89,151,159,280,742,757),]


# 混合效应模型
str(data)
data$Verb <- as.factor(data$Verb)

install.packages("lme4")
library(lme4)

model1<- glmer(Dative~ReciAnim
               +ReciAcc+ThemeAcc
               +ReciPron+ThemePron
               +LengthDiff
               +ReciDef+ThemeDef
               +ThemeCon
               +ThemeCom
               +(1|Verb),data=data,family=binomial)
summary(model1)

model2 <- glmer(Dative~ReciAnim
                +ReciAcc+ThemeAcc
                +ReciPron+ThemePron
                +LengthDiff
                +ReciDef+ThemeDef
                +ThemeCom
                +(1|Verb),data=data,family = binomial)
summary(model2)

anova(model1,model2,test = "Chisq")

predictions <- fitted(model2)
predictions.cat <- ifelse(predictions>0.5,"1","0")
table(data$Dative, predictions.cat)

attach(data)
Hmisc::somers2(fitted(model2),Dative == levels(Dative)[2])[1]

ranef(model2)$Verb
lattice::dotplot(ranef(model2, condVar = TRUE))
detach()

######### 多分类任务：德语中的三种回指选择
data <- read.csv("deAnap.csv",
  # 可以在读取数据的同时把所有分类变量批量转化为因子:
  stringsAsFactors=TRUE)

# 或者也可以手动逐个因子化、用lapply函数批量因子化
# data$Anapher <- as.factor(data$Anapher)
# data[ , 1:16] <- lapply(data[ , 1:16], as.factor)

str(data)


#########一对一比较（one vs. one）

# 使用`nnet`包
install.packages("nnet")
library(nnet)

## 数据处理

#### 设置代词回指（P）为因变量的参照水平
data$Anapher.ref<-relevel(data$Anapher,ref="P") 

#### 拟合最大模型
nn.maximal <-multinom(Anapher.ref~AnteSynF+AntePos+AnteTeilTyp+AnteForm+AnteSemR
                      +AnapSynF+AnapPos+AnapTeilTyp
                      +Konnektor+RefeDist+Belebtheit
                      +Topik+Mitbewerber+Textsorte+Parallelismus
                      +AnteTeilTyp*AnteSemR+AnapTeilTyp*AnteSemR
                      +Konnektor*AnteSemR
                      +Textsorte*AnteSynF,
                      data=data)
summary(nn.maximal)

#### 逐步回归、拟合最小模型
nn.minimal<-step(nn.maximal,direction = "both")
summary(nn.minimal)

#### 手动计算z-score和p-value
z<-summary(nn.minimal)$coefficients/summary(nn.minimal)$standard.errors
(p<-(1-pnorm(abs(z),0,1))*2)


#### 用自助法查看模型的拟合优度
fit <- multinom(Anapher.ref ~ AnteSynF+AnteTeilTyp+AnteSemR
                +AnapSynF+AnapPos+AnapTeilTyp
                +Konnektor+RefeDist+Belebtheit
                +Mitbewerber+Textsorte+Parallelismus
                +AnteTeilTyp*AnteSemR, data = data)

install.packages("boot")
library(boot)
bootstrap_fn <- function(data, indices) {
  fit <- multinom(Anapher.ref ~ AnteSynF+AnteTeilTyp+AnteSemR
                  +AnapSynF+AnapPos+AnapTeilTyp
                  +Konnektor+RefeDist+Belebtheit
                  +Mitbewerber+Textsorte+Parallelismus
                  +AnteTeilTyp*AnteSemR, 
                  data = data[indices, ])
  predictions <- predict(fit, newdata = data[indices, ])
  accuracy <- mean(predictions == data[indices, "Anapher.ref"])
  return(accuracy)
}

set.seed(1234)
results <- boot(data = data, statistic = bootstrap_fn, R = 1000)

bootstrap_accuracy <- results$t
original_accuracy <- mean(predict(fit, newdata = data) == data$Anapher.ref)

hist(results$t, main = "Bootstrap Accuracy")
abline(v = original_accuracy)

# 使用`polytomous`包（one vs. rest）
library(polytomous)

#### 拟合最大模型
pl.maximal<-polytomous(Anapher ~ AnteSynF+AntePos+AnteTeilTyp+AnteForm+AnteSemR
                          +AnapSynF+AnapPos+AnapTeilTyp
                          +Konnektor+RefeDist+Belebtheit
                          +Topik+Mitbewerber+Textsorte+Parallelismus
                          +AnteTeilTyp*AnteSemR+AnapTeilTyp*AnteSemR
                          +Konnektor*AnteSemR
                          +Textsorte*AnteSynF,
                          data=data)

#### 逐步回归
anova.polytomous(pl.maximal,statistic = "AIC")

#### 拟合最小模型
pl.minimal<-polytomous(Anapher ~ AnteSynF
                       +AnapSynF+AnapPos+AnapTeilTyp
                       +Konnektor+RefeDist+Belebtheit
                       +Topik+Mitbewerber+Textsorte+Parallelismus,
                       data=data)

#### 比较最大模型和最小模型是否具有显著差异
anova.polytomous(pl.minimal,pl.maximal)

#### 输出模型的统计结果
print(summary(pl.minimal),parameter="logodds",max.print=NA)
round(pl.minimal$p.values, 4) # 保留三位小数
summary(pl.minimal)$statistics


sessionInfo() #查看版本信息